'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function getExpressionsFromTemplateLiteral(node, t) {
  var raw = createRawStringFromTemplateLiteral(node);
  var minified = minify(raw);
  return replacePlaceholdersWithExpressions(minified, node.expressions || [], t);
}

var interleave = function interleave(strings, interpolations) {
  return interpolations.reduce(function (array, interp, i) {
    return array.concat([interp], strings[i + 1]);
  }, [strings[0]]);
};

function getDynamicMatches(str) {
  var re = /xxx(\d+)xxx/gm;
  var match;
  var matches = [];

  while ((match = re.exec(str)) !== null) {
    matches.push({
      value: match[0],
      p1: parseInt(match[1], 10),
      index: match.index
    });
  }

  return matches;
}

function _ref2(node) {
  return node.value !== '';
}

function replacePlaceholdersWithExpressions(str, expressions, t) {
  var matches = getDynamicMatches(str);

  if (expressions.length === 0) {
    if (str === '') {
      return [];
    }

    return [t.stringLiteral(str)];
  }

  var strings = [];
  var finalExpressions = [];
  var cursor = 0;
  matches.forEach(function (_ref, i) {
    var value = _ref.value,
        p1 = _ref.p1,
        index = _ref.index;
    var preMatch = str.substring(cursor, index);
    cursor = cursor + preMatch.length + value.length;

    if (preMatch) {
      strings.push(t.stringLiteral(preMatch));
    } else if (i === 0) {
      strings.push(t.stringLiteral(''));
    }

    finalExpressions.push(expressions[p1]);

    if (i === matches.length - 1) {
      strings.push(t.stringLiteral(str.substring(index + value.length)));
    }
  });
  return interleave(strings, finalExpressions).filter( // $FlowFixMe
  _ref2);
}

function _ref3(x) {
  return x.value.cooked;
}

function createRawStringFromTemplateLiteral(quasi) {
  var strs = quasi.quasis.map(_ref3);
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);

    if (i !== strs.length - 1) {
      arr.push("xxx" + i + "xxx");
    }

    return arr;
  }, []).join('').trim();
  return src;
} // babel-plugin-styled-components
// https://github.com/styled-components/babel-plugin-styled-components/blob/8d44acc36f067d60d4e09f9c22ff89695bc332d2/src/minify/index.js


var symbolRegex = /(\s*[;:{},]\s*)/g; // Counts occurences of substr inside str

var countOccurences = function countOccurences(str, substr) {
  return str.split(substr).length - 1;
};

function _ref4(str, fragment, index) {
  // Even-indices are non-symbol fragments
  if (index % 2 === 0) {
    return str + fragment;
  } // Only manipulate symbols outside of strings


  if (countOccurences(str, "'") % 2 === 0 && countOccurences(str, '"') % 2 === 0) {
    return str + fragment.trim();
  }

  return str + fragment;
}

var minify = function minify(code) {
  return code.split(symbolRegex).reduce(_ref4, '');
};

function getLabelFromPath(path, t) {
  return getIdentifierName(path, t);
}

function _ref(p) {
  return p.isVariableDeclarator();
}

function getDeclaratorName(path, t) {
  // $FlowFixMe
  var parent = path.findParent(_ref);
  return parent && t.isIdentifier(parent.node.id) ? parent.node.id.name : '';
}

function getIdentifierName(path, t) {
  var classParent;

  function _ref2(p) {
    return t.isClass(p);
  }

  if (path) {
    // $FlowFixMe
    classParent = path.findParent(_ref2);
  }

  if (classParent && classParent.node.id) {
    return t.isIdentifier(classParent.node.id) ? classParent.node.id.name : '';
  } else if (classParent && classParent.node.superClass && classParent.node.superClass.name) {
    return getDeclaratorName(path, t) + "(" + classParent.node.superClass.name + ")";
  }

  return getDeclaratorName(path, t);
}

exports.getExpressionsFromTemplateLiteral = getExpressionsFromTemplateLiteral;
exports.getLabelFromPath = getLabelFromPath;
//# sourceMappingURL=index.cjs.js.map
